"""
导入csv
导出csv或excel
"""
import csv
import datetime
from django import forms
from django.shortcuts import render, redirect, HttpResponse
from django.urls import path
from django.contrib import admin
import threading
import numpy as np
import pandas as pd
from bdtime import Time
from .django import get_model_max_id_in_db
from openpyxl import Workbook
import os


# --- 初始化环境 ---
if 1:
    root_path = '..' + os.sep + 'templates'
    root_path
    html_path_ls = 1
    os.path.exists(root_path)
    os.path.exists('../templates')
    os.path.exists('logs')
    print(__file__)

    1

class IDAdmin(admin.ModelAdmin):
    """
    * 保存时自动处理id, 解决postgre在批量导入数据后的主键冲突问题.

    - 该方法仅对admin界面手动保存时调用的save_model()方法生效, 不影响obj.save()方法效率.
    """

    def save_model(self, request, obj, form, change):
        """
        参数change分辨保存or修改.
        若为保存, 则model的id值自动更新为数据库中最大id+1.
        """
        if change is False:
            meta = obj._meta
            obj.id = get_model_max_id_in_db(model=None, meta=meta)
        obj.save()


# --- 处理df中的特殊格式
def conv_date_field_str_format(ts):
    """
    DateField的格式转换

    - 将csv中的时间字符转为符合django的时间格式
    """
    if not ts or ts == 'None':
        return None

    if isinstance(ts, float) and np.isnan(ts):
        return None
    try:
        ts = datetime.datetime.strptime(ts, '%Y/%m/%d')
        ts = datetime.datetime.strftime(ts, '%Y-%m-%d')
    except:
        try:
            # 若匹配得到， 说明格式不用换
            datetime.datetime.strptime(ts, '%Y-%m-%d')
        except:
            ts = None
    return ts


def format_time_column(df1, column_name):
    # 多行一起处理
    ts_ls = df1[column_name]
    ts_ls = [datetime.datetime.strptime(ts, '%Y/%m/%d') for ts in ts_ls]
    ts_ls = [datetime.datetime.strftime(ts, '%Y-%m-%d') for ts in ts_ls]
    df1[column_name] = ts_ls


def conv_nan(xx):
    # 特殊字符nan处理
    if xx == 'None' or (isinstance(xx, float) and np.isnan(xx)):
        return None
    else:
        return xx


class CsvImportForm(forms.Form):
    csv_file = forms.FileField()


class ExportExcelMixin:
    def export_as_excel(self, request, queryset=None, model=None):
        if model is None:
            meta = self.model._meta
        else:
            meta = model._meta

        # meta = self.model._meta
        field_names = [field.name for field in meta.fields]
        verbose_names = [field.verbose_name for field in meta.fields]

        response = HttpResponse(content_type='application/msexcel')
        response['Content-Disposition'] = f'attachment; filename={meta}.xlsx'
        wb = Workbook()
        ws = wb.active

        ws.append(verbose_names)
        for obj in queryset:
            data = [f'{getattr(obj, field)}' for field in field_names]
            ws.append(data)
        wb.save(response)
        return response

    export_as_excel.short_description = "导出所选数据到Excel"
    export_as_excel.acts_on_all = True
    # export_as_excel.type = 'success'
    # export_as_excel.icon = 'el-icon-upload'
    export_as_excel.icon = 'el-icon-download'


class ExportCsvMixin:
    def export_as_csv(self, request, queryset, model=None):
        if model is None:
            meta = self.model._meta
        else:
            meta = model._meta
        field_names = [field.name for field in meta.fields]
        verbose_names = [field.verbose_name for field in meta.fields]

        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename={}.csv'.format(meta)
        writer = csv.writer(response)
        writer.writerow(verbose_names)
        for obj in queryset:
            writer.writerow([getattr(obj, field) for field in field_names])
        return response

    export_as_csv.short_description = "  导出选中数据"        # 有图标的话要空俩格, 不然太近了
    export_as_csv.acts_on_all = True
    # export_as_csv.icon = 'fas fa-audio-description'
    # export_as_csv.icon = 'fas fa-download'
    export_as_csv.icon = 'fas fa-download'


# class ShowListPerPageMixin:
#     def show_list_per_page(self, request, queryset, model=None):
#         if model is None:
#             meta = self.model._meta
#         else:
#             meta = model._meta
#         print(1)
#         return redirect('.')
#
#     show_list_per_page.short_description = "展示数据"

class ImportAdmin(IDAdmin):
    """
    导入类, CSV和Excel通用

    - 不能与admin.ModelAdmin一起用!
    """
    change_list_template = "entities/mychange_list.html"

    def import_csv(self, request):
        try:
            if request.method == "POST":
                csv_file = request.FILES.get("csv_file")
                assert csv_file, '文件不能为空!'
                assert csv_file._name and csv_file._name.__contains__('.'), '文件名不能为空, 且必须有后缀名!'
                f_format = csv_file._name.rsplit('.', 1)[-1]
                format_ls = ['xls', 'xlsx', 'csv']
                assert f_format in format_ls, f'不支持的文件类型! 目前仅支持{format_ls}.'

                read_data = csv_file.read()

                time_str = datetime.datetime.now().strftime("%Y%m%d_%H%M%S")

                tempdir = 'tempdir'
                if not os.path.exists(tempdir):
                    os.mkdir(tempdir)

                if f_format == 'csv':
                    try:
                        encoding = 'utf-8'
                        file_data = read_data.decode(encoding)
                    except Exception as e:
                        print('-- 尝试用gbk编码 --')
                        encoding = 'gbk'
                        file_data = read_data.decode(encoding)

                    fname = f'f_{time_str}.csv'
                    fname = os.path.join(tempdir, fname)

                    with open(fname, 'w', encoding=encoding) as f:
                        f.write(file_data)

                    # 为解决字段内有逗号导致分割错误问题, 只能采用pd了
                    df = pd.read_csv(fname, encoding=encoding)
                elif f_format in ['xlsx', 'xls']:
                    import xlrd
                    wb = xlrd.open_workbook(file_contents=read_data)
                    df = pd.read_excel(wb)

                    # sheet_0 = wb.sheets()[0]
                    # row = sheet_0.nrows
                    # header = sheet_0.row_values(0)
                    # print(header)
                    # for i in range(1, row):
                    #     col = sheet_0.row_values(i)
                    #     print(col)
                else:
                    df = None

                titles = df.columns.tolist()

                meta = self.model._meta
                field_names = [field.name for field in meta.fields]
                verbose_names = [field.verbose_name for field in meta.fields]
                field_dc = dict(zip(verbose_names, field_names))

                title_ls = [field_dc.get(title_i) if field_dc.get(title_i) else title_i for title_i in titles]

                curr_id = get_model_max_id_in_db(self.model)

                df1 = df.copy()
                df1.columns = title_ls

                for index, row in df1.iterrows():
                    content_ls = row.values.tolist()

                    # 处理DateField字段
                    for i in range(len(title_ls)):
                        title_i = title_ls[i]
                        content_ls[i] = conv_nan(content_ls[i])

                        attr = getattr(self.model, title_i)
                        if not hasattr(attr, 'field'):
                            continue
                        attr_field_name = attr.field.__class__.__name__

                        # 只解决了DateField, 还缺DateTimeField待解决.
                        if attr_field_name in ['DateField', 'DateTimeField']:
                            # print(i, title_i, content_ls[i])
                            content_ls[i] = conv_date_field_str_format(content_ls[i])

                    dc = dict(zip(title_ls, content_ls))
                    dc.update({'id': curr_id})
                    curr_id += 1

                    self.model.objects.create(**dc)

                self.message_user(request, f"{f_format}文件导入成功!")
                self.remove_temp_file(tempdir)
                return redirect("..")
        except Exception as e:
            self.message_user(request, "导入失败! 错误信息: " + str(e))
            return redirect(".")

        form = CsvImportForm()
        payload = {"form": form}
        return render(request, "admin/csv_form.html", payload)

    def export_all_csv(self, request):
        # self.message_user(request, "成功导出全部数据为csv文件")
        return ExportCsvMixin().export_as_csv(request, queryset=self.model.objects.all(), model=self.model)

    def export_all_excel(self, request):
        # self.message_user(request, "成功导出全部数据为csv文件")
        return ExportExcelMixin().export_as_excel(request, queryset=self.model.objects.all(), model=self.model)

    # def show_list_per_page(self, request):
    #     pass

    def get_urls(self):
        my_urls = [
            path('import-csv/', self.import_csv),
            path('export_all_csv/', self.export_all_csv),
            path('export_all_excel/', self.export_all_excel),
            # path('show_list_per_page/', self.show_list_per_page),
        ]
        return my_urls + super().get_urls()

    def remove_temp_file(self, tempdir):
        # 内存清理
        temps = len(os.listdir(tempdir))
        MAX_TEMPS = 30
        if temps > MAX_TEMPS:
            t1 = threading.Thread(target=ImportAdmin._remove_temp_file, args=(tempdir, MAX_TEMPS, ))
            t1.start()
        else:
            print(f'...excel缓存还足够, 不用清理... 缓存容量: {temps}/{MAX_TEMPS}')
        return

    @staticmethod
    def _remove_temp_file(tempdir, MAX_TEMPS=5):
        # --- 清理缓存, 清空tempdir下的所有文件
        fpath_ls = os.listdir(tempdir)
        temps = len(fpath_ls)

        if temps < MAX_TEMPS:
            print(f'...缓存还足够, 不用清理... 缓存容量: {temps}/{MAX_TEMPS}')
            return
        tt = Time()

        tt.sleep(1)
        print('*************** 开始清理缓存 *************')
        for fpath in fpath_ls:
            i = 0
            tt.__init__()
            while tt.during(5):
                i += 1
                dirpath = os.path.join(tempdir, fpath)
                try:
                    os.remove(dirpath)
                    print(f"~~~ success: 移除文件[{dirpath}]成功! -- 第[{i}]次")
                    break
                except:
                    print(f"** 第[{i}]次移除文件[{dirpath}]失败...")
                    tt.sleep(1)
                    if i > 5:
                        print(f"======== Warning: 移除文件[{dirpath}]失败!")
        print('*************** 缓存清理完毕 *************')


class CsvImportExportAdmin(ImportAdmin, ExportCsvMixin):
    """
    CSV导入/导出Admin类

    - 不能与admin.ModelAdmin一起用!
    """

    actions = ['export_as_csv']


class ExcelImportExportAdmin(ImportAdmin, ExportExcelMixin):
    """
    Excel导入/导出Admin类

    - 不能与admin.ModelAdmin一起用!
    """

    actions = ['export_as_excel']

    def get_search_results(self, request, queryset, search_term):
        """
        * 若search_term以变量prefix的值开头, 则检索最近xx条记录.

        - 如'~10'代表检索最近10条记录
        """
        prefix = '~'
        if search_term is not None and not search_term.startswith(prefix):
            ret = super().get_search_results(request, queryset, search_term)
        else:
            try:
                if search_term.startswith(prefix):
                    search_term = search_term
                    search_term = int(search_term[len(prefix):])
            except:
                raise TypeError('id必须为整数!')
            id_qsv_ls = queryset.values('id')[:search_term]
            queryset = queryset.filter(id__in=id_qsv_ls)
            ret = (queryset, False)
        return ret




"""
纯python的功能函数
"""
import json
import pandas as pd


def f1():
    print('1')


def conv_queryset_ls_to_serialzer_ls(qs: list):
    """
    queryset数据的序列化
    """
    dcls = []
    if not qs:
        return dcls

    q = qs[0]
    kname_ls = list(q.keys())

    # 大于二维的数据序列化
    if kname_ls.__len__() > 2:
        for q in qs:
            dcls.append(q)
        return dcls
    else:
        kname, vname = kname_ls

    # 二维数据序列化
    for q in qs:
        k, v = q.get(kname), q.get(vname)
        dc = {
            kname: k,
            vname: v
        }
        dcls.append(dc)
    return dcls


def add_status_and_msg(dc_ls, status=200, msg=None):
    if status != 200 and msg is None:
        msg = '请求数据失败!'

    if status == 200 and msg is None:
        msg = "ok"

    ret = {
        'status': status,
        'msg': msg,
        'result': dc_ls
    }
    return ret


def show_json(data: dict):
    try:
        print(json.dumps(data, sort_keys=True, indent=4, separators=(', ', ': '), ensure_ascii=False))
    except:
        for k, v in data:
            print(k, ' --- ', v)


def add_space_prefix(text, n, more=True, prefix='\u3000'):
    text = str(text)
    if more:
        ret = prefix * n + text
    else:
        ret = prefix * (n - len(text)) + text
    return ret


def conv_df_to_serializer_data(df: pd.DataFrame, add_index=False) -> list:
    assert isinstance(df, pd.DataFrame), 'df的类型必须是DataFrame!'
    ret_ls = []
    for index, row in df.iterrows():
        k = row.index.tolist()
        v = row.values.tolist()
        data = dict(zip(k, v))

        if add_index:
            # 将df的index作为dc_ls的key
            data = {
                'name': index,
                'value': data
            }

        ret_ls.append(data)
    return ret_ls



"""
依赖django的功能函数
"""

from . import pure

import math
import pandas as pd

from django.core.paginator import Paginator
from rest_framework.response import Response
from rest_framework.pagination import PageNumberPagination
from rest_framework.generics import GenericAPIView
from rest_framework.mixins import ListModelMixin, RetrieveModelMixin
from rest_framework.request import Request
from functools import wraps
from django.db import connection
from django.db.models import Q
from django.db.models import QuerySet


def APIResponse(ret, status=200, msg=None):
    if isinstance(ret, Response):
        ret = ret.data
    ret = pure.add_status_and_msg(ret, status=status, msg=msg)
    ret = Response(ret)
    return ret


class Pagination(PageNumberPagination):
    """
    * 默认分页器参数设置

    - page_size: 每页16个
    - page_size_query_param: 前端控制每页数量时使用的参数名, 'page_size'
    - page_query_param: 页码控制参数名"p"
    - max_page_size: 最大100页
    """
    page_size = 100
    page_size_query_param = 'page_size'
    page_query_param = 'p'
    max_page_size = 1000


class BaseList(ListModelMixin, RetrieveModelMixin, GenericAPIView):
    """
    * API: BaseModel的ListView和RetrieveView接口

    - 测试接口:
        - List:
            GET /api/index/BaseList/?order_type=-id&page_size=4&p=1
        - Retrieve:
            GET /api/index/BaseList/5/
    """

    pagination_class = Pagination
    filter_fields = []

    def get(self, request: Request, *args, **kwargs):
        """
        - 如果request携带pk参数, 则进行Retrieve操作, 否则进行List操作.

        - BaseList的默认get请求参数(仅在List操作时生效)
            - page_size: 分页器每页数量, 前端用来控制数据每页展示的数量, 在Pagination类中设置.
            - p: 第p页.
            - order_type: 排序字段, 如"id"和"-id".
        """
        if kwargs.get('pk'):
            ret, status, msg = self.get_retrieve_ret(request, *args, **kwargs)
        else:
            ret, status, msg = self.get_list_ret(request, *args, **kwargs)
        return APIResponse(ret, status=status, msg=msg)

    def get_retrieve_ret(self, request: Request, *args, **kwargs):
        """
        Retrieve操作

        - pk必须在`url.py::urlpatterns`中设置, 如: path('BaseList/<str:pk>/', views.BaseList.as_view())
        """
        status = 200
        msg = 'ok'

        try:
            ret = self.retrieve(request)
        except Exception as e:
            # 没找到的情况, 404 Not Found
            ret = None
            status = 404
            msg = str(e)
        return ret, status, msg

    def get_list_ret(self, request: Request, *args, **kwargs):
        """
        List操作
        """
        status = 200
        msg = 'ok'

        self._get_list_queryset()

        # --- 解析请求参数
        query_dc = request.query_params

        order_type = query_dc.get('order_type')
        if order_type:
            try:
                self.queryset = self.queryset.order_by(order_type)
            except ValueError as e:
                ret = None
                status = 404
                msg = f'参数order_type指定的排序字段[{order_type}]取值错误!'
                return ret, status, msg

        # --- 获取返回数据
        try:
            ret = self.list(request)
        except Exception as e:
            # 页码无效的情况, 404 Not Found
            ret = None
            status = 404
            msg = str(e)
        return ret, status, msg

    def _get_list_queryset(self):
        """
        得到queryset, 仅对list方法生效
        """
        query_dc = self.request.query_params
        self.queryset = super().get_queryset()
        if not isinstance(self.queryset, QuerySet):
            self.queryset = self.queryset.objects.all()

        if self.queryset.count():
            """
            过滤字段filter_fields
            """
            meta = self.queryset[0]._meta
            field_names = [field.name for field in meta.fields]
            for fn in field_names:
                if fn in self.filter_fields:
                    if fn in query_dc:
                        value = query_dc.get(fn)
                        exec(f"self.queryset = self.queryset.filter(Q({fn}='{value}'))")

        return self.queryset

    def list(self, request: Request, *args, **kwargs):
        resp = super(BaseList, self).list(request, *args, **kwargs)
        ret = self._conv_data_format(resp)
        return ret

    def _conv_data_format(self, data: (dict, Response)):
        if isinstance(data, Response):
            data = data.data

        # 分页信息
        count = data.get('count')
        page_size = self.request.query_params.get('page_size', self.pagination_class.page_size)
        p = self.request.query_params.get('p', 1)
        total = math.ceil(count / int(page_size))
        page_dc = {
            'count': count,
            'total': total,
            'page_size': page_size,
            'p': p,
        }

        results = data.get('results')

        ret = {
            'page_dc': page_dc,
            'results': results,
        }
        return ret


def paginate_qsls_to_dcls(qsls, serializer, page: int, per_page=16):
    """
    * 手动分页函数

    - 指定模型的queryset_ls和serializer, 然后按给定的page和per_page参数获取分页后的数据
    """

    p = Paginator(qsls, per_page)
    page_obj = p.get_page(page)
    page_dc = {
        'num_pages': p.num_pages,
        'count_objects': p.count,
        'current_page_number': page_obj.number,
    }

    # --- 处理单个Model和多个Model的情况
    if serializer.__class__.__name__ == 'function':
        dc_ls = serializer(page_obj)
    else:
        dc_ls = serializer(page_obj, many=True).data
    return dc_ls, page_dc


def conv_qsls_to_dcls(qsls: list, names: list=None, new_names=None):
    """
    将一个queryset_list转换为dict_list
    """
    if not qsls:
        return []

    if names is None:
        names = list(qsls[0].keys())

    if not new_names:
        new_names = names

    dc_ls = []
    for qs in qsls:
        if names and new_names:
            # 优化版
            values = []
            for name in names:
                values.append(qs.get(name))

            dc = dict(zip(new_names, values))
        else:
            # 手动一算
            if names.__len__() == 2:
                name, value = qs.get(names[0]), qs.get(names[1])
                dc = {
                    new_names[0]: name,
                    new_names[1]: value,
                }
            elif names.__len__() == 3:
                name, code, value = qs.get(names[0]), qs.get(names[1]), qs.get(names[2])
                dc = {
                    new_names[0]: name,
                    new_names[1]: code,
                    new_names[2]: value,
                }
            else:
                print('长度不对!')
                dc = {}
        dc_ls.append(dc)
    return dc_ls


def order_qs_ls_by_id(qs_ls, sort_by='id'):
    df = pd.DataFrame(qs_ls).sort_values(by=sort_by)

    cols = df.columns
    dc_ls = []
    for i, row in df.iterrows():
        dc = {
            cols[0]: row.get(cols[0]),
            cols[1]: row.get(cols[1]),
        }
        dc_ls.append(dc)
    return dc_ls


def api_decorator(func):
    """
    * API装饰器

    - 如果运行出错, 将格式化输出错误的信息, 并返回给前端, 而不会报错.
    """
    @wraps(func)
    def wrapped_function(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            print('--- API Error! ---')
            print(e)
            return APIResponse(str(e), status=404, msg='Error!')
    return wrapped_function


def get_model_max_id_in_db(model=None, meta=None):
    """
    仅适用于postgresql, mysql直接忽略就行.
    """
    if not meta:
        meta = model._meta

    cursor = connection.cursor()
    # db_prefix = meta.__str__().split('.')[0]

    # --- 先尝试创建id_seq
    id_seq = f"{meta.app_label}_{meta.db_table}_id_seq"

    try:
        sql = f"""CREATE SEQUENCE IF NOT EXISTS {id_seq}"""
        cursor.execute(sql)
    except Exception as e:
        print(e)
        print('不是PostgreSQL无法运行CREATE SEQUENCE语句! 请确认数据库类型!')

    # --- 找出最大的id
    sql = f"""select setval('{id_seq}', (select max(id) from "{meta.db_table}")+1);"""
    # sql = '(select max(id) from  "{meta.db_table}")'
    print('sql---', sql)
    cursor.execute(sql)
    row = cursor.fetchall()
    curr_id = row[0][0]
    ret = 0 if curr_id is None else curr_id
    return ret




import os
import shutil
# --- 初始化环境 ---


def create_dir_if_not_exist(dirpath: str):
    if not os.path.exists(dirpath):
        os.mkdir(dirpath)
        return False
    return True


def run():
    create_dir_if_not_exist('templates')
    create_dir_if_not_exist(os.path.join('templates', 'admin'))
    create_dir_if_not_exist(os.path.join('templates', 'entities'))

    src1 = os.path.join(os.path.dirname(__file__), os.path.join(os.path.join('templates', 'admin'), 'csv_form.html'))
    dst1 = os.path.join('templates', 'admin')
    shutil.copy2(src1, dst1)

    src = os.path.join(os.path.dirname(__file__), os.path.join(os.path.join('templates', 'entities'), 'mychange_list.html'))
    dst = os.path.join('templates', 'entities')
    shutil.copy2(src, dst)


if __name__ == '__main__':
    run()


"""
高级检索
"""

from .django import BaseList, APIResponse
from django.db.models import Q
from django.db.models import QuerySet


class SingleConditionSearch:
    """
    # 单一条件检索

    - 示例:
    condition = {
        "add_logic": "and",
        "search_method": "academy",
        "search_keywords": "人文艺术",
        "accuracy": "0"
    }
    """
    model = None
    serializer = None
    qs = Q()

    def __init__(self, condition: dict):
        self.condition = condition
        self.search_type, self.search_method, self.search_keywords = condition.get(
            'add_logic'), condition.get(
            'search_method'), condition.get('search_keywords')
        self.accuracy = int(condition.get('accuracy', 0))
        self.qs = Q()
        self.queryset = None

    def add_field_as_accuracy(self):
        """
        根据accuracy的值, 增加field_name
        :param search_field: 字段名
        :param search_keywords: 字段值
        :param accuracy: 精确检索 or 模糊检索s
        :return: qs
        """
        search_field, search_keywords, accuracy = self.search_method, self.search_keywords, self.accuracy

        assert self.model is not None, '请指定model类型!'

        if isinstance(self.model, QuerySet):
            assert self.model.count(), '请指定检索的queryset类型!'
            model = self.model[0]
        else:
            model = self.model
        if model._meta.get_field(search_field).get_internal_type() in ('TextField', 'CharField'):
            if accuracy:
                cmd = f'self.qs = Q({search_field}="{search_keywords}")'
            else:
                cmd = f'self.qs = Q({search_field}__contains="{search_keywords}")'
        else:
            cmd = f'self.qs = Q({search_field}={search_keywords})'
        exec(cmd)

        qs = self.qs
        # print(search_field, search_keywords, accuracy, '---', qs)
        return qs

    def get_q(self):
        """
        重点
        :return: 单个条件的检索逻辑
        """
        qs = self.add_field_as_accuracy()
        return qs


class MultipleConditionSearch:
    """
    # 多重条件检索(高级检索)

    - 示例:
    ```python
    search_condition_ls = [
            {
                "add_logic": "and",
                "search_method": "academy",
                "search_keywords": "人文艺术",
                "accuracy": "0"
            },
            {
                "add_logic": "and",
                "search_method": "name",
                "search_keywords": "博",
                "accuracy": "0"
            }
        ]

    mcs = MultipleConditionSearch(MySingleConditionSearch, search_condition_ls)
    mcs.add_multiple_conditions()
    mcs.QS
    queryset = mcs.get_queryset()
    ```

    """

    def __init__(self, model, condition_ls: list):
        self.QS = Q()
        self.queryset = []
        self.condition_ls = condition_ls        # 检索条件
        # if isinstance(model, QuerySet):
        #     assert model.count(), '请指定检索的queryset类型!'
        #     self.model = model[0]
        # else:
        self.model = model

        class MySingleConditionSearch(SingleConditionSearch):
            model = self.model

        self.SingleConditionSearch = MySingleConditionSearch      # 检索类型
        assert MySingleConditionSearch.model is not None, '请指定SingleConditionSearch的model类型!'

    def add_q(self, q, add_type: str):
        return MultipleConditionSearch.qs_add(self.QS, q, add_type)

    @staticmethod
    def qs_add(qs, q, add_type: str):
        """
        高级检索的条件拼接
        """
        if add_type == 'not':
            qs.add(~q, Q.AND)
        elif add_type == 'and':
            qs.add(q, Q.AND)
        elif add_type == 'or':
            qs.add(q, Q.OR)
        else:
            raise ValueError('qs_add: add_logic取值错误!')

        return qs

    def add_single_condition(self, condition: dict):
        """
        按单一条件补充QS
        """
        scs = self.SingleConditionSearch(condition)
        q = scs.get_q()
        return self.add_q(q, scs.search_type)

    def add_multiple_conditions(self):
        """
        按条件列表补充QS
        """
        condition_ls = self.condition_ls
        if not condition_ls or isinstance(condition_ls[0], list):
            return self.QS

        for condition in condition_ls:
            self.add_single_condition(condition)

        return self.QS

    def get_queryset(self):
        if isinstance(self.model, QuerySet):
            self.queryset = self.model.filter(self.QS)
        else:
            self.queryset = self.model.objects.filter(self.QS)
        return self.queryset


class AdvancedSearchView(BaseList):
    """
    高级检索

    POST /api/index/AdvancedSearch
    """
    queryset = None
    serializer_class = None
    search_condition_ls = None

    def post(self, request, *args, **kwargs):
        ret, status, msg = self.get_list_ret(request, *args, **kwargs)
        return APIResponse(ret, status=status, msg=msg)

    def get_queryset(self):
        import json

        if self.search_condition_ls is None:
            search_condition_ls = self.request.data.get('search_condition_ls', [])
            if not search_condition_ls:
                search_condition_ls = self.request.query_params.get('search_condition_ls', [])
                if isinstance(search_condition_ls, str):
                    search_condition_ls = json.loads(search_condition_ls)
        else:
            search_condition_ls = self.search_condition_ls

        if search_condition_ls:
            mcs = MultipleConditionSearch(self.queryset, search_condition_ls)
            mcs.add_multiple_conditions()
            self.queryset = mcs.get_queryset()
        else:
            self.queryset = super().get_queryset()

        return self.queryset


import setuptools
import os


dirname = 'bddjango'
templates_name_ls = [
    # dirname + os.sep + 'templates',
    dirname + os.sep + 'templates' + os.sep + 'ttt.py',
    # dirname + os.sep + 'templates' + os.sep + 'entities',
    dirname + os.sep + 'templates' + os.sep + 'admin' + os.sep + 't1.py',
    dirname + os.sep + 'templates' + os.sep + 'admin' + os.sep + 'csv_form.html',
    dirname + os.sep + 'templates' + os.sep + 'entities' + os.sep + 'mychange_list.html',
]


with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()


setuptools.setup(
    name=dirname,
    version="0.0.5",
    author="bode135",
    author_email='2248270222@qq.com',   # 作者邮箱
    description="常用的django开发工具",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=f'https://github.com/bode135/{dirname}',   # 主页链接
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=['pandas', ],      # 依赖模块
    data_files=[('', templates_name_ls)],
)
